@@ -1,6 +1,6 @@
 package Chemistry::Mol;
-$VERSION = '0.34';
-# $Id: Mol.pm,v 1.47 2005/05/16 23:29:12 itubert Exp $
+$VERSION = '0.36';
+# $Id: Mol.pm,v 1.49 2005/09/20 14:06:50 itubert Exp $
 
 =head1 NAME
 
@@ -498,7 +498,7 @@ Read a file and return a list of Mol objects, or croaks if there was a problem.
 The type of file will be guessed if not specified via the C<format> option.
 
 Note that only registered file readers will be used. Readers may be registered
-using C<register_type()>; modules that include readers (such as
+using C<register_format()>; modules that include readers (such as
 L<Chemistry::File::PDB>) usually register them automatically when they are
 loaded.
 
@@ -587,7 +587,7 @@ following is a hypothetical example for reading MDL rxnfiles.
     $yield     = $rxn->yield;     # a number
 
 Note that only registered file readers will be used. Readers may be registered
-using register_type(); modules that include readers (such as
+using register_format(); modules that include readers (such as
 Chemistry::File::PDB) usually register them automatically.
 
 =cut
@@ -637,7 +637,7 @@ sub register_format {
 =item Chemistry::Mol->formats
 
 Returns a list of the file formats that have been installed by
-register_type()
+register_format()
 
 =cut
 
@@ -733,7 +733,8 @@ has a pointer to the rest of the universe, the entire universe will be cloned!
 sub clone {
     my ($self) = @_;
     my $clone = dclone $self;
-    $clone->_weaken;
+    $clone->_weaken if Storable->VERSION < 2.14;
+    $clone;
 }
 
 =item my $mol2 = $mol->safe_clone;
@@ -909,13 +910,40 @@ sub collapse_hydrogens {
     }
 }
 
+=item $mol->add_implicit_hydrogens
+
+Use heuristics to figure out how many implicit hydrogens should each atom in 
+the molecule have to satisfy its normal "organic" valence.
+
+=cut
+
+sub add_implicit_hydrogens {
+    my ($self) = @_;
+    $_->add_implicit_hydrogens for $self->atoms;
+}
+
+
+my %DESCRIPTORS  = ();
+
+sub descriptor {
+    my ($self, $descriptor) = @_;
+    my $sub = $DESCRIPTORS{$descriptor}
+        or croak "unknown descriptor '$descriptor'";
+    return $sub->($self);
+}
+
+sub register_descriptor {
+    my ($self, %opts) = @_;
+    $DESCRIPTORS{$_} = $opts{$_} for keys %opts;
+}
+
 1;
 
 =back
 
 =head1 VERSION
 
-0.34
+0.36
 
 =head1 SEE ALSO
 
